@@ -1,10 +1,10 @@
-__version__ = "0.1.5"
+__version__ = "0.1.6"
 __git_commiter_name__ = "Arnon Yaari"
 __git_commiter_email__ = "arnony@infinidat.com"
-__git_branch__ = '(Not currently on any branch)'
+__git_branch__ = '(detached from v0.1.6)'
 __git_remote_tracking_branch__ = '(No remote tracking)'
 __git_remote_url__ = '(Not remote tracking)'
-__git_head_hash__ = '4f0bc25283414e6be5e8d3d2d23e3873edbf87a0'
-__git_head_subject__ = 'Finished release v0.1.5.'
+__git_head_hash__ = '2dcda1208ae79d34d273502a05155b3ca61df3b4'
+__git_head_subject__ = 'Finished release v0.1.6.'
 __git_head_message__ = ''
 __git_dirty_diff__ = ''
\ No newline at end of file
